@@ -493,8 +493,10 @@ public final class Params {
   }
 
   public void reconcile() {
-    if (scale.get() != 100 && desktopSize.getMode() == DesktopSize.AUTO) {
-      vlog.info("Desktop scaling enabled.  Disabling automatic desktop resizing.");
+    if ((scale.get() == ScaleParameter.AUTO ||
+         scale.get() == ScaleParameter.FIXEDRATIO) &&
+        desktopSize.getMode() == DesktopSize.AUTO) {
+      vlog.info("Automatic desktop scaling enabled.  Disabling automatic desktop resizing.");
       desktopSize.setMode(DesktopSize.SERVER);
     }
   }
@@ -847,7 +849,7 @@ public final class Params {
   "the entire image will fit in the viewer window without using " +
   "scrollbars.  If this parameter is set to \"FixedRatio\", then automatic " +
   "scaling is performed, but the original aspect ratio is preserved.  " +
-  "Enabling scaling disables automatic desktop resizing.", 100);
+  "Enabling automatic scaling disables automatic desktop resizing.", 100);
 
   public SpanParameter span =
   new SpanParameter("Span", this,
@@ -1016,10 +1018,7 @@ public final class Params {
   (Utils.getBooleanProperty("turbovnc.sessmgr", true) ?
    "When using the TurboVNC Session Manager, this parameter is effectively " +
    "disabled.  (The built-in SSH client is always used with the TurboVNC " +
-   "Session Manager.)\n " : "") +
-
-  "This parameter is effectively set if the Server parameter specifies a " +
-  "Unix domain socket connection to a remote host.", false);
+   "Session Manager.)" : ""), false);
 
   public StringParameter extSSHCommand =
   new StringParameter("ExtSSHCommand", this, false, true,
