@@ -60,7 +60,7 @@ class OptionsDialog extends Dialog implements ActionListener, ChangeListener,
   private JButton x509caButton, x509crlButton;
   private JLabel x509caLabel, x509crlLabel;
   private JTextField x509ca, x509crl;
-  private JButton listClearButton, defClearButton, resetButton;
+  private JButton clearHistoryButton, deleteOptionsButton, resetButton;
   private JLabel encMethodLabel;
   private JLabel jpegQualityLabel, jpegQualityLabelLo, jpegQualityLabelHi;
   private JLabel subsamplingLabel, subsamplingLabelLo, subsamplingLabelHi;
@@ -69,6 +69,18 @@ class OptionsDialog extends Dialog implements ActionListener, ChangeListener,
   private Hashtable<Integer, String> subsamplingLabelTable;
   private String oldScalingFactor, oldDesktopSize;
   private boolean enableX509 = true;
+  private static final String[] SCALING_FACTORS = {
+    "Auto", "Fixed Aspect Ratio", "50%", "75%", "95%", "100%", "105%", "125%",
+    "150%", "175%", "200%", "250%", "300%", "350%", "400%"
+  };
+  private static final String[] SCALING_FACTORS_ADR = {
+    "50%", "75%", "95%", "100%", "105%", "125%", "150%", "175%", "200%",
+    "250%", "300%", "350%", "400%"
+  };
+  private static final DefaultComboBoxModel<String> CB_MODEL =
+    new DefaultComboBoxModel<String>(SCALING_FACTORS);
+  private static final DefaultComboBoxModel<String> CB_MODEL_ADR =
+    new DefaultComboBoxModel<String>(SCALING_FACTORS_ADR);
 
   OptionsDialog(OptionsDialogCallback callback_, Params params_) {
     super(true);
@@ -247,11 +259,7 @@ class OptionsDialog extends Dialog implements ActionListener, ChangeListener,
     displayPanel.setBorder(BorderFactory.createTitledBorder("Display"));
 
     JLabel scalingFactorLabel = new JLabel("Scaling factor:");
-    Object[] scalingFactors = {
-      "Auto", "Fixed Aspect Ratio", "50%", "75%", "95%", "100%", "105%",
-      "125%", "150%", "175%", "200%", "250%", "300%", "350%", "400%"
-    };
-    scalingFactor = new JComboBox(scalingFactors);
+    scalingFactor = new JComboBox(CB_MODEL);
     scalingFactor.setEditable(true);
     scalingFactor.addItemListener(this);
 
@@ -499,19 +507,19 @@ class OptionsDialog extends Dialog implements ActionListener, ChangeListener,
     // Global tab
     globalPanel = new JPanel(new GridBagLayout());
 
-    listClearButton = new JButton("Clear the list of saved connections");
-    listClearButton.addActionListener(this);
-    defClearButton = new JButton("Clear all saved per-host options");
-    defClearButton.addActionListener(this);
+    clearHistoryButton = new JButton("Clear the connection history");
+    clearHistoryButton.addActionListener(this);
+    deleteOptionsButton = new JButton("Delete saved options for all hosts");
+    deleteOptionsButton.addActionListener(this);
     resetButton = new JButton("Reset all options to defaults");
     resetButton.addActionListener(this);
 
-    Dialog.addGBComponent(listClearButton, globalPanel,
+    Dialog.addGBComponent(clearHistoryButton, globalPanel,
                           0, 0, 2, 1, 2, 2, 1, 0,
                           GridBagConstraints.NONE,
                           GridBagConstraints.FIRST_LINE_START,
                           new Insets(8, 5, 0, 5));
-    Dialog.addGBComponent(defClearButton, globalPanel,
+    Dialog.addGBComponent(deleteOptionsButton, globalPanel,
                           0, 1, 2, 1, 2, 2, 1, 0,
                           GridBagConstraints.NONE,
                           GridBagConstraints.FIRST_LINE_START,
@@ -839,10 +847,10 @@ class OptionsDialog extends Dialog implements ActionListener, ChangeListener,
       endDialog();
     } else if (s instanceof JButton && (JButton)s == cancelButton) {
       endDialog();
-    } else if (s instanceof JButton && (JButton)s == listClearButton) {
-      UserPreferences.clear(true);
-    } else if (s instanceof JButton && (JButton)s == defClearButton) {
-      UserPreferences.clear(false);
+    } else if (s instanceof JButton && (JButton)s == clearHistoryButton) {
+      UserPreferences.clearHistory();
+    } else if (s instanceof JButton && (JButton)s == deleteOptionsButton) {
+      UserPreferences.deleteAllOptions();
     } else if (s instanceof JButton && (JButton)s == resetButton) {
       Params oldParams = params;
       params = new Params();
@@ -979,11 +987,15 @@ class OptionsDialog extends Dialog implements ActionListener, ChangeListener,
         oldDesktopSize = newsize;
         if (!newsize.equals(newDesktopSize))
           desktopSize.setSelectedItem(newsize);
+        String sf = scalingFactor.getSelectedItem().toString();
         if (size.getMode() == DesktopSize.AUTO) {
-          scalingFactor.setEnabled(false);
-          scalingFactor.setSelectedItem("100%");
+          if (sf.equalsIgnoreCase("Fixed Aspect Ratio") ||
+              sf.equalsIgnoreCase("Auto"))
+            sf = "100%";
+          scalingFactor.setModel(CB_MODEL_ADR);
         } else
-          scalingFactor.setEnabled(true);
+          scalingFactor.setModel(CB_MODEL);
+        scalingFactor.setSelectedItem(sf);
       }
     }
     if (s instanceof JCheckBox && (JCheckBox)s == tunnel) {
@@ -1291,9 +1303,9 @@ class OptionsDialog extends Dialog implements ActionListener, ChangeListener,
 
     desktopSize.setEnabled(enableDesktopSize);
     if (params.desktopSize.getMode() == DesktopSize.AUTO)
-      scalingFactor.setEnabled(false);
+      scalingFactor.setModel(CB_MODEL_ADR);
     else
-      scalingFactor.setEnabled(true);
+      scalingFactor.setModel(CB_MODEL);
     if (disableShared) shared.setEnabled(false);
     updateSecurityPanel();
     if (disableSecurity) {
