@@ -271,10 +271,10 @@ public final class VncViewer implements Runnable, OptionsDialogCallback {
       //    subsequent calls to getInsets().
       // Thus, we have to compute the insets globally using a dummy JFrame.
       // Dear Swing, eff ewe.
-      final JFrame frame = new JFrame();
-      // Under certain Linux WM's, the insets aren't valid until
-      // componentResized() is called (see above.)
       if (Utils.isX11()) {
+        // Under certain Linux WMs, the insets aren't valid until
+        // componentResized() is called (see above.)
+        final JFrame frame = new JFrame();
         frame.addComponentListener(new ComponentAdapter() {
           public void componentResized(ComponentEvent e) {
             synchronized (frame) {
@@ -293,12 +293,20 @@ public final class VncViewer implements Runnable, OptionsDialogCallback {
             frame.wait();
         }
         frame.setVisible(false);
-      } else {
+        frame.dispose();
+      }
+      // With newer versions of GNOME (and only with newer versions of GNOME),
+      // the method above returns insets with all values set to 0, but the
+      // cross-platform method below works.
+      if (!Utils.isX11() ||
+          (insets != null && insets.top == 0 && insets.left == 0 &&
+           insets.bottom == 0 && insets.right == 0)) {
+        JFrame frame = new JFrame();
         frame.setVisible(true);
         insets = frame.getInsets();
         frame.setVisible(false);
+        frame.dispose();
       }
-      frame.dispose();
     } catch (Exception e) {
       vlog.error("Could not set insets:");
       vlog.error("  " + e.toString());
@@ -351,6 +359,7 @@ public final class VncViewer implements Runnable, OptionsDialogCallback {
 
   static {
     setBlitterDefaults();
+    Params.loadSystemProperties();
   }
 
   static void startViewer(String[] argv) {
@@ -485,6 +494,12 @@ public final class VncViewer implements Runnable, OptionsDialogCallback {
           continue;
         }
 
+        if (argv[i].equals("-??")) {
+          System.out.print("\nThe advanced parameters are:\n\n");
+          params.list(80, true);
+          System.exit(1);
+        }
+
         if (params.set(argv[i]))
           continue;
 
@@ -583,6 +598,9 @@ public final class VncViewer implements Runnable, OptionsDialogCallback {
       "      connection info file but not parameter values specified on the command\n" +
       "      line after the connection info file.\n" +
       "\n" +
+      "  -??\n" +
+      "      List rarely-used advanced parameters and their descriptions.\n" +
+      "\n" +
       "Specifying boolean parameters:\n" +
       "  On:   -<param> or --<param> or <param>=1 or -<param>=1 or --<param>=1\n" +
       "  Off:  -no<param> or --no<param> or <param>=0 or -<param>=0 or --<param>=0\n" +
@@ -601,7 +619,7 @@ public final class VncViewer implements Runnable, OptionsDialogCallback {
     System.out.println("Copyright (C) " + copyrightYear + " " + copyright);
     System.out.println(url);
     System.out.print(usage);
-    params.list(80);
+    params.list(80, false);
     System.exit(1);
   }
 
@@ -651,12 +669,6 @@ public final class VncViewer implements Runnable, OptionsDialogCallback {
 
   public void start() {
     vlog.debug("start called");
-    String host = params.server.get();
-    int port = params.port.get();
-    if (host != null && Hostname.getColonPos(host) < 0 && port > 0) {
-      params.server.set(host + ((port >= 5900 && port <= 5999) ?
-                                (":" + (port - 5900)) : ("::" + port)));
-    }
     nViewers++;
     thread = new Thread(this);
     thread.start();
