@@ -512,6 +512,14 @@ int ddxProcessArgument(int argc, char *argv[], int i)
   }
 #endif
 
+  if (strcasecmp(argv[i], "-drinode") == 0) {
+#ifdef DRI3
+    REQUIRE_ARG();
+    driNode = argv[i + 1];
+#endif
+    return 2;
+  }
+
   if (strcasecmp(argv[i], "-geometry") == 0) {
     /* -geometry WxH or W0xH0+X0+Y0[,W1xH1+X1+Y1,...] */
     char *str, *token;
@@ -991,6 +999,11 @@ static Bool rfbScreenInit(ScreenPtr pScreen, int argc, char **argv)
   if (prfb->bitsPerPixel > 4)
     fbPictureInit(pScreen, 0, 0);
 
+#ifdef DRI3
+  if (driNode && !rfbDRI3Initialize(pScreen))
+    return FALSE;
+#endif
+
   if (!dixRegisterPrivateKey(&rfbGCKey, PRIVATE_GC, sizeof(rfbGCRec)))
     FatalError("rfbScreenInit: dixRegisterPrivateKey failed");
 
@@ -998,6 +1011,10 @@ static Bool rfbScreenInit(ScreenPtr pScreen, int argc, char **argv)
   prfb->dontSendFramebufferUpdate = FALSE;
 
   prfb->CloseScreen = pScreen->CloseScreen;
+#ifdef DRI3
+  if (driNode)
+    prfb->SourceValidate = pScreen->SourceValidate;
+#endif
   prfb->CreateGC = pScreen->CreateGC;
   prfb->CopyWindow = pScreen->CopyWindow;
   prfb->ClearToBackground = pScreen->ClearToBackground;
@@ -1006,6 +1023,15 @@ static Bool rfbScreenInit(ScreenPtr pScreen, int argc, char **argv)
   if (ps) {
     prfb->Composite = ps->Composite;
     prfb->Glyphs = ps->Glyphs;
+#ifdef DRI3
+    if (driNode) {
+      prfb->CompositeRects = ps->CompositeRects;
+      prfb->Trapezoids = ps->Trapezoids;
+      prfb->Triangles = ps->Triangles;
+      prfb->TriStrip = ps->TriStrip;
+      prfb->TriFan = ps->TriFan;
+    }
+#endif
   }
 #endif
   prfb->InstallColormap = pScreen->InstallColormap;
@@ -1015,6 +1041,10 @@ static Bool rfbScreenInit(ScreenPtr pScreen, int argc, char **argv)
   prfb->SaveScreen = pScreen->SaveScreen;
 
   pScreen->CloseScreen = rfbCloseScreen;
+#ifdef DRI3
+  if (driNode)
+    pScreen->SourceValidate = rfbSourceValidate;
+#endif
   pScreen->CreateGC = rfbCreateGC;
   pScreen->CopyWindow = rfbCopyWindow;
   pScreen->ClearToBackground = rfbClearToBackground;
@@ -1022,6 +1052,15 @@ static Bool rfbScreenInit(ScreenPtr pScreen, int argc, char **argv)
   if (ps) {
     ps->Composite = rfbComposite;
     ps->Glyphs = rfbGlyphs;
+#ifdef DRI3
+    if (driNode) {
+      ps->CompositeRects = rfbCompositeRects;
+      ps->Trapezoids = rfbTrapezoids;
+      ps->Triangles = rfbTriangles;
+      ps->TriStrip = rfbTriStrip;
+      ps->TriFan = rfbTriFan;
+    }
+#endif
   }
 #endif
   pScreen->InstallColormap = rfbInstallColormap;
@@ -1259,15 +1298,15 @@ Bool AddExtInputDevice(rfbDevInfo *dev)
       dev->pDev->xinput_type = MakeAtom("PAD", strlen("PAD"), 1);
       break;
     default:
-      if (stristr(dev->name, "cursor"))
+      if (strcasestr(dev->name, "cursor"))
         dev->pDev->xinput_type = XA_CURSOR;
-      else if (stristr(dev->name, "stylus"))
+      else if (strcasestr(dev->name, "stylus"))
         dev->pDev->xinput_type = MakeAtom("STYLUS", strlen("STYLUS"), 1);
-      else if (stristr(dev->name, "eraser"))
+      else if (strcasestr(dev->name, "eraser"))
         dev->pDev->xinput_type = MakeAtom("ERASER", strlen("ERASER"), 1);
-      else if (stristr(dev->name, "touch"))
+      else if (strcasestr(dev->name, "touch"))
         dev->pDev->xinput_type = MakeAtom("TOUCH", strlen("TOUCH"), 1);
-      else if (stristr(dev->name, "pad"))
+      else if (strcasestr(dev->name, "pad"))
         dev->pDev->xinput_type = MakeAtom("PAD", strlen("PAD"), 1);
       break;
   }
@@ -1824,7 +1863,12 @@ void ddxUseMsg(void)
   ErrorF("========================\n");
   ErrorF("-depth D               set framebuffer depth\n");
 #ifndef TURBOVNC_STATIC_XORG_PATHS
-  ErrorF("-dridir dir            specify directory containing the swrast Mesa driver\n");
+  ErrorF("-dridir dir            specify directory containing the Mesa DRI drivers\n");
+#endif
+#ifdef DRI3
+  ErrorF("-drinode render-node   enable the DRI3 X extension with the GPU corresponding to\n");
+  ErrorF("                       the specified DRM render node (open source GPU drivers\n");
+  ErrorF("                       only)\n");
 #endif
   ErrorF("-geometry WxH          set framebuffer width & height (single-screen)\n");
   ErrorF("-geometry W0xH0+X0+Y0[,W1xH1+X1+Y1,...,WnxHn+Xn+Yn]\n");
