@@ -4,8 +4,8 @@
  *
  */
 
-/* Copyright (C) 2014-2016, 2019, 2021-2022, 2024 D. R. Commander.
- *                                                All Rights Reserved.
+/* Copyright (C) 2014-2016, 2019, 2021-2022, 2024-2025 D. R. Commander.
+ *                                                     All Rights Reserved.
  * Copyright (C) 2013, 2017-2018 Pierre Ossman for Cendio AB.
  *                               All Rights Reserved.
  * Copyright (C) 2009 Red Hat, Inc.  All Rights Reserved.
@@ -526,7 +526,7 @@ void PtrAddEvent(int buttonMask, int x, int y)
     cursorPosY = y;
   }
 
-  for (i = 0; i < 7; i++) {
+  for (i = 0; i < 9; i++) {
     if ((buttonMask ^ oldButtonMask) & (1 << i)) {
       if (buttonMask & (1 << i)) {
         valuator_mask_set_range(&mask, 0, 0, NULL);
@@ -545,27 +545,6 @@ void PtrAddEvent(int buttonMask, int x, int y)
 }
 
 
-Bool stristr(const char *s1, const char *s2)
-{
-  char *str1, *str2, *ret;
-  int i;
-
-  if (!s1 || !s2 || strlen(s1) < 1 || strlen(s2) < 1)
-    return FALSE;
-
-  str1 = strdup(s1);
-  for (i = 0; i < strlen(str1); i++)
-    str1[i] = tolower(str1[i]);
-  str2 = strdup(s2);
-  for (i = 0; i < strlen(str2); i++)
-    str2[i] = tolower(str2[i]);
-
-  ret = strstr(str1, str2);
-  free(str1);  free(str2);
-  return ret != NULL;
-}
-
-
 void ExtInputAddEvent(rfbDevInfoPtr dev, int type, int buttons)
 {
   ValuatorMask mask;
@@ -591,13 +570,13 @@ void ExtInputAddEvent(rfbDevInfoPtr dev, int type, int buttons)
         vtDev = &virtualTabletPad;
         break;
       default:
-        if (stristr(dev->name, "stylus"))
+        if (strcasestr(dev->name, "stylus"))
           vtDev = &virtualTabletStylus;
-        else if (stristr(dev->name, "eraser"))
+        else if (strcasestr(dev->name, "eraser"))
           vtDev = &virtualTabletEraser;
-        else if (stristr(dev->name, "touch"))
+        else if (strcasestr(dev->name, "touch"))
           vtDev = &virtualTabletTouch;
-        else if (stristr(dev->name, "pad"))
+        else if (strcasestr(dev->name, "pad"))
           vtDev = &virtualTabletPad;
         else
           return;
